@@ -1,5 +1,5 @@
 /* nbdkit
- * Copyright (C) 2014-2019 Red Hat Inc.
+ * Copyright (C) 2014-2020 Red Hat Inc.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -42,6 +42,9 @@
 #include <string.h>
 #include <unistd.h>
 #include <fcntl.h>
+#include <sys/stat.h>
+
+#include <pthread.h>
 
 #include <libssh/libssh.h>
 #include <libssh/sftp.h>
@@ -49,6 +52,9 @@
 
 #include <nbdkit-plugin.h>
 
+#include "array-size.h"
+#include "cleanup.h"
+#include "const-string-vector.h"
 #include "minmax.h"
 
 static const char *host = NULL;
@@ -58,10 +64,12 @@ static const char *user = NULL;
 static char *password = NULL;
 static bool verify_remote_host = true;
 static const char *known_hosts = NULL;
-static const char **identity = NULL;
-static size_t nr_identities = 0;
+static const_string_vector identities = empty_vector;
 static uint32_t timeout = 0;
 static bool compression = false;
+static bool create = false;
+static int64_t create_size = -1;
+static unsigned create_mode = S_IRUSR | S_IWUSR /* 0600 */;
 
 /* config can be:
  * NULL => parse options from default file
@@ -71,7 +79,7 @@ static bool compression = false;
 static const char *config = NULL;
 
 /* Use '-D ssh.log=N' to set. */
-int ssh_debug_log = 0;
+NBDKIT_DLL_PUBLIC int ssh_debug_log = 0;
 
 /* If ssh_debug_log > 0 then the library will call this function with
  * log messages.
@@ -81,10 +89,9 @@ log_callback (int priority, const char *function, const char *message, void *vp)
 {
   const char *levels[] =
     { "none", "warning", "protocol", "packet", "function" };
-  const size_t nr_levels = sizeof levels / sizeof levels[0];
   const char *level;
 
-  if (priority >= 0 && priority < nr_levels)
+  if (priority >= 0 && priority < ARRAY_SIZE (levels))
     level = levels[priority];
   else
     level = "unknown";
@@ -98,7 +105,7 @@ log_callback (int priority, const char *function, const char *message, void *vp)
 static void
 ssh_unload (void)
 {
-  free (identity);
+  free (identities.ptr);
   free (password);
 }
 
@@ -133,16 +140,11 @@ ssh_config (const char *key, const char *value)
     known_hosts = value; /* %-expanded, cannot use nbdkit_absolute_path */
 
   else if (strcmp (key, "identity") == 0) {
-    const char **new_identity =
-      realloc (identity, (nr_identities+1) * sizeof (const char *));
-    if (new_identity == NULL) {
+    /* %-expanded, cannot use nbdkit_absolute_path on value */
+    if (const_string_vector_append (&identities, value) == -1) {
       nbdkit_error ("realloc: %m");
       return -1;
     }
-    identity = new_identity;
-    /* %-expanded, cannot use nbdkit_absolute_path */
-    identity[nr_identities] = value;
-    nr_identities++;
   }
 
   else if (strcmp (key, "verify-remote-host") == 0) {
@@ -171,6 +173,27 @@ ssh_config (const char *key, const char *value)
       return -1;
     compression = r;
   }
+  else if (strcmp (key, "create") == 0) {
+    r = nbdkit_parse_bool (value);
+    if (r == -1)
+      return -1;
+    create = r;
+  }
+  else if (strcmp (key, "create-size") == 0) {
+    create_size = nbdkit_parse_size (value);
+    if (create_size == -1)
+      return -1;
+  }
+  else if (strcmp (key, "create-mode") == 0) {
+    r = nbdkit_parse_unsigned (key, value, &create_mode);
+    if (r == -1)
+      return -1;
+    /* OpenSSH checks this too. */
+    if (create_mode > 0777) {
+      nbdkit_error ("create-mode must be <= 0777");
+      return -1;
+    }
+  }
 
   else {
     nbdkit_error ("unknown parameter '%s'", key);
@@ -190,6 +213,13 @@ ssh_config_complete (void)
     return -1;
   }
 
+  /* If create=true, create-size must be supplied. */
+  if (create && create_size == -1) {
+    nbdkit_error ("if using create=true, you must specify the size "
+                  "of the new remote file using create-size=SIZE");
+    return -1;
+  }
+
   return 0;
 }
 
@@ -204,7 +234,15 @@ ssh_config_complete (void)
   "identity=<FILENAME>        Prepend private key (identity) file.\n" \
   "timeout=SECS               Set SSH connection timeout.\n" \
   "verify-remote-host=false   Ignore known_hosts.\n" \
-  "compression=true           Enable compression."
+  "compression=true           Enable compression.\n" \
+  "create=true                Create the remote file.\n" \
+  "create-mode=MODE           Set the permissions of the remote file.\n" \
+  "create-size=SIZE           Set the size of the remote file."
+
+/* Since we must simulate atomic pread and pwrite using seek +
+ * read/write, calls on each handle must be serialized.
+ */
+#define THREAD_MODEL NBDKIT_THREAD_MODEL_SERIALIZE_REQUESTS
 
 /* The per-connection handle. */
 struct ssh_handle {
@@ -220,24 +258,6 @@ static int
 do_verify_remote_host (struct ssh_handle *h)
 {
   enum ssh_known_hosts_e state;
-  unsigned char *hash = NULL;
-  ssh_key srv_pubkey = NULL;
-  size_t hlen;
-  int rc;
-
-  rc = ssh_get_server_publickey (h->session, &srv_pubkey);
-  if (rc < 0) {
-    nbdkit_error ("could not get server public key");
-    return -1;
-  }
-  rc = ssh_get_publickey_hash (srv_pubkey,
-                               SSH_PUBLICKEY_HASH_SHA1,
-                               &hash, &hlen);
-  ssh_key_free (srv_pubkey);
-  if (rc < 0) {
-    nbdkit_error ("could not get server public key SHA1 hash");
-    return -1;
-  }
 
   state = ssh_session_is_known_server (h->session);
   switch (state) {
@@ -247,13 +267,11 @@ do_verify_remote_host (struct ssh_handle *h)
 
   case SSH_KNOWN_HOSTS_CHANGED:
     nbdkit_error ("host key for server changed");
-    ssh_clean_pubkey_hash (&hash);
     return -1;
 
   case SSH_KNOWN_HOSTS_OTHER:
     nbdkit_error ("host key for server was not found "
                   "but another type of key exists");
-    ssh_clean_pubkey_hash (&hash);
     return -1;
 
   case SSH_KNOWN_HOSTS_NOT_FOUND:
@@ -261,22 +279,18 @@ do_verify_remote_host (struct ssh_handle *h)
      * host key is set up before using nbdkit so we error out here.
      */
     nbdkit_error ("could not find known_hosts file");
-    ssh_clean_pubkey_hash (&hash);
     return -1;
 
   case SSH_KNOWN_HOSTS_UNKNOWN:
     nbdkit_error ("host key is unknown, you must use ssh first "
                   "and accept the host key");
-    ssh_clean_pubkey_hash (&hash);
     return -1;
 
   case SSH_KNOWN_HOSTS_ERROR:
     nbdkit_error ("known hosts error: %s", ssh_get_error (h->session));
-    ssh_clean_pubkey_hash (&hash);
     return -1;
   }
 
-  ssh_clean_pubkey_hash (&hash);
   return 0;
 }
 
@@ -320,6 +334,19 @@ authenticate (struct ssh_handle *h)
     return -1;
 
   method = ssh_userauth_list (h->session, NULL);
+  nbdkit_debug ("authentication methods offered by the server [0x%x]: "
+                "%s%s%s%s%s%s%s",
+                method,
+                method & SSH_AUTH_METHOD_NONE        ? " none" : "",
+                method & SSH_AUTH_METHOD_PASSWORD    ? " password" : "",
+                method & SSH_AUTH_METHOD_PUBLICKEY   ? " publickey" : "",
+                method & SSH_AUTH_METHOD_HOSTBASED   ? " hostbased" : "",
+                method & SSH_AUTH_METHOD_INTERACTIVE
+                       ? " keyboard-interactive" : "",
+                method & SSH_AUTH_METHOD_GSSAPI_MIC
+                       ? " gssapi-with-mic" : "",
+                method & ~0x3f
+                       ? " (and other unknown methods)" : "");
 
   if (method & SSH_AUTH_METHOD_PUBLICKEY) {
     rc = authenticate_pubkey (h->session);
@@ -339,6 +366,65 @@ authenticate (struct ssh_handle *h)
   return -1;
 }
 
+/* This function opens or creates the remote file (depending on
+ * create=false|true).  Parallel connections might call this function
+ * at the same time, and so we must hold a lock to ensure that the
+ * file is created at most once.
+ */
+static pthread_mutex_t create_lock = PTHREAD_MUTEX_INITIALIZER;
+
+static sftp_file
+open_or_create_path (ssh_session session, sftp_session sftp, int readonly)
+{
+  ACQUIRE_LOCK_FOR_CURRENT_SCOPE (&create_lock);
+  int access_type;
+  int r;
+  sftp_file file;
+
+  access_type = readonly ? O_RDONLY : O_RDWR;
+  if (create) access_type |= O_CREAT | O_TRUNC;
+
+  file = sftp_open (sftp, path, access_type, S_IRWXU);
+  if (!file) {
+    nbdkit_error ("cannot %s file for %s: %s",
+                  create ? "create" : "open",
+                  readonly ? "reading" : "writing",
+                  ssh_get_error (session));
+    return NULL;
+  }
+
+  if (create) {
+    /* There's no sftp_truncate call.  However OpenSSH lets you call
+     * SSH_FXP_SETSTAT + SSH_FILEXFER_ATTR_SIZE which invokes
+     * truncate(2) on the server.  Libssh doesn't provide a binding
+     * for SSH_FXP_FSETSTAT so we have to pass the session + path.
+     */
+    struct sftp_attributes_struct attrs = {
+      .flags = SSH_FILEXFER_ATTR_SIZE |
+               SSH_FILEXFER_ATTR_PERMISSIONS,
+      .size = create_size,
+      .permissions = create_mode,
+    };
+
+    r = sftp_setstat (sftp, path, &attrs);
+    if (r != SSH_OK) {
+      nbdkit_error ("setstat failed: %s", ssh_get_error (session));
+
+      /* Best-effort attempt to delete the remote file on failure. */
+      r = sftp_unlink (sftp, path);
+      if (r != SSH_OK)
+        nbdkit_debug ("unlink failed: %s", ssh_get_error (session));
+
+      return NULL;
+    }
+  }
+
+  /* On the next connection, don't create or truncate the file. */
+  create = false;
+
+  return file;
+}
+
 /* Create the per-connection handle. */
 static void *
 ssh_open (int readonly)
@@ -347,7 +433,6 @@ ssh_open (int readonly)
   const int set = 1;
   size_t i;
   int r;
-  int access_type;
 
   h = calloc (1, sizeof *h);
   if (h == NULL) {
@@ -410,11 +495,12 @@ ssh_open (int readonly)
      * as this file is rarely present.
      */
   }
-  for (i = 0; i < nr_identities; ++i) {
-    r = ssh_options_set (h->session, SSH_OPTIONS_ADD_IDENTITY, identity[i]);
+  for (i = 0; i < identities.len; ++i) {
+    r = ssh_options_set (h->session,
+                         SSH_OPTIONS_ADD_IDENTITY, identities.ptr[i]);
     if (r != SSH_OK) {
       nbdkit_error ("failed to add identity in libssh session: %s: %s",
-                    identity[i], ssh_get_error (h->session));
+                    identities.ptr[i], ssh_get_error (h->session));
       goto err;
     }
   }
@@ -480,7 +566,7 @@ ssh_open (int readonly)
   if (authenticate (h) == -1)
     goto err;
 
-  /* Open the SFTP connection and file. */
+  /* Open the SFTP connection. */
   h->sftp = sftp_new (h->session);
   if (!h->sftp) {
     nbdkit_error ("failed to allocate sftp session: %s",
@@ -493,14 +579,11 @@ ssh_open (int readonly)
                   ssh_get_error (h->session));
     goto err;
   }
-  access_type = readonly ? O_RDONLY : O_RDWR;
-  h->file = sftp_open (h->sftp, path, access_type, S_IRWXU);
-  if (!h->file) {
-    nbdkit_error ("cannot open file for %s: %s",
-                  readonly ? "reading" : "writing",
-                  ssh_get_error (h->session));
+
+  /* Open or create the remote file. */
+  h->file = open_or_create_path (h->session, h->sftp, readonly);
+  if (!h->file)
     goto err;
-  }
 
   nbdkit_debug ("opened libssh handle");
 
@@ -536,8 +619,6 @@ ssh_close (void *handle)
   free (h);
 }
 
-#define THREAD_MODEL NBDKIT_THREAD_MODEL_SERIALIZE_REQUESTS
-
 /* Get the file size. */
 static int64_t
 ssh_get_size (void *handle)
@@ -625,6 +706,19 @@ ssh_can_flush (void *handle)
   return sftp_extension_supported (h->sftp, "fsync@openssh.com", "1");
 }
 
+static int
+ssh_can_multi_conn (void *handle)
+{
+  struct ssh_handle *h = handle;
+
+  /* After examining the OpenSSH implementation of sftp-server we
+   * concluded that its write/flush behaviour is safe for advertising
+   * multi-conn.  Other servers may not be safe.  Use the
+   * fsync@openssh.com feature as a proxy.
+   */
+  return sftp_extension_supported (h->sftp, "fsync@openssh.com", "1");
+}
+
 static int
 ssh_flush (void *handle)
 {
@@ -658,6 +752,7 @@ static struct nbdkit_plugin plugin = {
   .pwrite            = ssh_pwrite,
   .can_flush         = ssh_can_flush,
   .flush             = ssh_flush,
+  .can_multi_conn    = ssh_can_multi_conn,
 };
 
 NBDKIT_REGISTER_PLUGIN(plugin)
