@@ -34,6 +34,7 @@ source ./functions.sh
 set -e
 
 requires_filter log
+requires qemu-io --version
 
 sock1=$(mktemp -u /tmp/nbdkit-test-sock.XXXXXX)
 sock2=$(mktemp -u /tmp/nbdkit-test-sock.XXXXXX)
@@ -42,17 +43,14 @@ files="blocksize1.img blocksize1.log $sock1 blocksize1.pid
 rm -f $files
 
 # Prep images, and check that qemu-io understands the actions we plan on doing.
-# TODO: Until we implement NBD_INFO_BLOCK_SIZE, qemu-io does its own
-# read-modify-write at 512-byte alignment, while we'd like to ultimately test
-# 1-byte accesses
-truncate -s 10M blocksize1.img
+$TRUNCATE -s 10M blocksize1.img
 if ! qemu-io -f raw -c 'r 0 1' -c 'w -z 1000 2000' \
      -c 'w -P 0 1M 2M' -c 'discard 3M 4M' blocksize1.img; then
     echo "$0: missing or broken qemu-io"
     rm blocksize1.img
     exit 77
 fi
-truncate -s 10M blocksize2.img
+$TRUNCATE -s 10M blocksize2.img
 
 # For easier debugging, dump the final log files before removing them
 # on exit.
