@@ -38,7 +38,7 @@ set -e
 set -x
 
 requires_plugin eval
-requires nbdinfo --version
+requires_nbdinfo
 requires_nbdsh_uri
 requires nbdsh -c 'print(h.set_full_info)'
 requires jq --version
