@@ -1,5 +1,5 @@
 /* nbdkit
- * Copyright (C) 2018-2020 Red Hat Inc.
+ * Copyright (C) 2018-2021 Red Hat Inc.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -72,6 +72,13 @@ test_layers_plugin_config_complete (void)
 
 #define test_layers_plugin_config_help "test_layers_plugin_config_help"
 
+static int
+test_layers_plugin_thread_model (void)
+{
+  DEBUG_FUNCTION;
+  return NBDKIT_THREAD_MODEL_PARALLEL;
+}
+
 static int
 test_layers_plugin_get_ready (void)
 {
@@ -86,6 +93,12 @@ test_layers_plugin_after_fork (void)
   return 0;
 }
 
+static void
+test_layers_plugin_cleanup (void)
+{
+  DEBUG_FUNCTION;
+}
+
 static int
 test_layers_plugin_preconnect (int readonly)
 {
@@ -93,6 +106,21 @@ test_layers_plugin_preconnect (int readonly)
   return 0;
 }
 
+static int
+test_layers_plugin_list_exports (int readonly, int default_only,
+                                 struct nbdkit_exports *exports)
+{
+  DEBUG_FUNCTION;
+  return nbdkit_add_export (exports, "", NULL);
+}
+
+static const char *
+test_layers_plugin_default_export (int readonly, int is_tls)
+{
+  DEBUG_FUNCTION;
+  return "";
+}
+
 static void *
 test_layers_plugin_open (int readonly)
 {
@@ -150,6 +178,13 @@ test_layers_plugin_can_zero (void *handle)
   return 1;
 }
 
+static int
+test_layers_plugin_can_fast_zero (void *handle)
+{
+  DEBUG_FUNCTION;
+  return 1;
+}
+
 static int
 test_layers_plugin_can_fua (void *handle)
 {
@@ -245,9 +280,13 @@ static struct nbdkit_plugin plugin = {
   .config            = test_layers_plugin_config,
   .config_complete   = test_layers_plugin_config_complete,
   .config_help       = test_layers_plugin_config_help,
+  .thread_model      = test_layers_plugin_thread_model,
   .get_ready         = test_layers_plugin_get_ready,
   .after_fork        = test_layers_plugin_after_fork,
+  .cleanup           = test_layers_plugin_cleanup,
   .preconnect        = test_layers_plugin_preconnect,
+  .list_exports      = test_layers_plugin_list_exports,
+  .default_export    = test_layers_plugin_default_export,
   .open              = test_layers_plugin_open,
   .close             = test_layers_plugin_close,
   .get_size          = test_layers_plugin_get_size,
@@ -256,6 +295,7 @@ static struct nbdkit_plugin plugin = {
   .is_rotational     = test_layers_plugin_is_rotational,
   .can_trim          = test_layers_plugin_can_trim,
   .can_zero          = test_layers_plugin_can_zero,
+  .can_fast_zero     = test_layers_plugin_can_fast_zero,
   .can_fua           = test_layers_plugin_can_fua,
   .can_multi_conn    = test_layers_plugin_can_multi_conn,
   .can_extents       = test_layers_plugin_can_extents,
