@@ -1,5 +1,5 @@
 /* nbdkit
- * Copyright (C) 2018-2020 Red Hat Inc.
+ * Copyright (C) 2018-2021 Red Hat Inc.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -59,7 +59,6 @@
 
 #include "byte-swapping.h"
 #include "cleanup.h"
-#include "exit-with-parent.h"
 
 /* Declare program_name. */
 #if HAVE_DECL_PROGRAM_INVOCATION_SHORT_NAME == 1
@@ -96,16 +95,17 @@ main (int argc, char *argv[])
 {
   struct nbd_handle *nbd;
   int pfd[2];
+  int r;
   int err;
   pthread_t thread;
   int orig_stderr;
   char data[512];
 
-#ifndef HAVE_EXIT_WITH_PARENT
-  printf ("%s: this test requires --exit-with-parent functionality\n",
-          program_name);
-  exit (77);
-#endif
+  if (system ("nbdkit --exit-with-parent --version") != 0) {
+    printf ("%s: this test requires --exit-with-parent functionality\n",
+            program_name);
+    exit (77);
+  }
 
   /* Prepare libnbd. */
   fprintf (stderr, "%s: beginning test\n", program_name);
@@ -128,7 +128,13 @@ main (int argc, char *argv[])
   /* Start a thread which will just listen on the pipe and
    * place the log messages in a memory buffer.
    */
-  if (pipe2 (pfd, O_CLOEXEC) == -1) {
+#ifdef HAVE_PIPE2
+  r = pipe2 (pfd, O_CLOEXEC);
+#else
+  /* Just ignore the O_CLOEXEC requirement, it's only a test. */
+  r = pipe (pfd);
+#endif
+  if (r == -1) {
     perror ("pipe2");
     exit (EXIT_FAILURE);
   }
@@ -157,19 +163,20 @@ main (int argc, char *argv[])
   }
 
   /* Start nbdkit. */
-  char *args[] = {
-    "nbdkit", "--exit-with-parent", "-fvns",
-    /* Because of asynchronous shutdown with threads, finalize
-     * isn't reliably called unless we disable parallel.
-     */
-    "-t", "1",
-    "--filter", ".libs/test-layers-filter3." SOEXT,
-    "--filter", ".libs/test-layers-filter2." SOEXT,
-    "--filter", ".libs/test-layers-filter1." SOEXT,
-    ".libs/test-layers-plugin." SOEXT,
-    "foo=bar",
-    NULL};
-  if (nbd_connect_command (nbd, args) == -1) {
+  if (nbd_connect_command (nbd,
+                           (char *[]) {
+                             "nbdkit", "--exit-with-parent", "-fvns",
+                             /* Because of asynchronous shutdown with
+                              * threads, finalize isn't reliably
+                              * called unless we disable parallel.
+                              */
+                             "-t", "1",
+                             "--filter", ".libs/test-layers-filter3." SOEXT,
+                             "--filter", ".libs/test-layers-filter2." SOEXT,
+                             "--filter", ".libs/test-layers-filter1." SOEXT,
+                             ".libs/test-layers-plugin." SOEXT,
+                             "foo=bar",
+                             NULL }) == -1) {
     dprintf (orig_stderr, "nbd_connect_command: %s\n", nbd_get_error ());
     exit (EXIT_FAILURE);
   }
@@ -234,28 +241,28 @@ main (int argc, char *argv[])
      "filter3: test_layers_filter_thread_model",
      NULL);
 
-  /* get_ready methods called in order. */
+  /* get_ready methods called in inner-to-outer order. */
   log_verify_seen_in_order
-    ("testlayersfilter3: get_ready",
-     "filter3: test_layers_filter_get_ready",
-     "testlayersfilter2: get_ready",
-     "filter2: test_layers_filter_get_ready",
+    ("testlayersplugin: get_ready",
+     "test_layers_plugin_get_ready",
      "testlayersfilter1: get_ready",
      "filter1: test_layers_filter_get_ready",
-     "testlayersplugin: get_ready",
-     "test_layers_plugin_get_ready",
+     "testlayersfilter2: get_ready",
+     "filter2: test_layers_filter_get_ready",
+     "testlayersfilter3: get_ready",
+     "filter3: test_layers_filter_get_ready",
      NULL);
 
-  /* after_fork methods called in order. */
+  /* after_fork methods called in inner-to-outer order. */
   log_verify_seen_in_order
-    ("testlayersfilter3: after_fork",
-     "filter3: test_layers_filter_after_fork",
-     "testlayersfilter2: after_fork",
-     "filter2: test_layers_filter_after_fork",
+    ("testlayersplugin: after_fork",
+     "test_layers_plugin_after_fork",
      "testlayersfilter1: after_fork",
      "filter1: test_layers_filter_after_fork",
-     "testlayersplugin: after_fork",
-     "test_layers_plugin_after_fork",
+     "testlayersfilter2: after_fork",
+     "filter2: test_layers_filter_after_fork",
+     "testlayersfilter3: after_fork",
+     "filter3: test_layers_filter_after_fork",
      NULL);
 
   /* preconnect methods called in outer-to-inner order, complete
@@ -609,6 +616,18 @@ main (int argc, char *argv[])
      "test_layers_plugin_close",
      NULL);
 
+  /* cleanup methods called in outer-to-inner order. */
+  log_verify_seen_in_order
+    ("testlayersfilter3: cleanup",
+     "filter3: test_layers_filter_cleanup",
+     "testlayersfilter2: cleanup",
+     "filter2: test_layers_filter_cleanup",
+     "testlayersfilter1: cleanup",
+     "filter1: test_layers_filter_cleanup",
+     "testlayersplugin: cleanup",
+     "test_layers_plugin_cleanup",
+     NULL);
+
   /* unload methods should be run in any order. */
   log_verify_seen ("test_layers_plugin_unload");
   log_verify_seen ("filter1: test_layers_filter_unload");
