@@ -41,12 +41,13 @@ requires_nbdsh_uri
 # - Test quoting.
 # - size=0 because we ignore it in the command itself.
 nbdkit -f -v -U - tmpdisk 0 a=2 a=1 b=1024 c="a ' b ' c" \
+       TRUNCATE="$TRUNCATE" \
        command='
 set -x
 set -e
 if [ $a -ne 1 ]; then exit 1; fi
 if [ "$c" != "a '\'' b '\'' c" ]; then exit 1; fi
-truncate -s $b "$disk"
+$TRUNCATE -s $b "$disk"
 ' \
        --run '
 nbdsh -u "$uri" -c "assert h.get_size() == 1024"
